import java.awt.*;
import java.util.*;
/**
  *
  * Objekte der Klasse stellen ein Listenelement des Hauptspeichers dar.
  * Ein Listenobjekt wird ber einen Zeiger oder eine Kette von anderen Listen-
  * elementen referenziert. Je nachdem, ob das Listenelement direkt ber einen
  * Zeiger oder indirekt ber andere Listenelemente referenziert wird, wird
  * es unterschiedlich dargestellt.
  *
  * @version 1.0 vom 23.11.2013
  * @author Daniel Garmann
  */

public class Listenobjekt extends Hauptspeicherobjekt {
  
  // Anfang Attribute
  public static final int DIREKTERREICHBAR = 1;   // ber einen Zeiger erreichbar
  public static final int INDIREKTERREICHBAR = 2; // ber ein Listenobjekt erreichbar, welches an einem Zeiger hngt.
  public static final int ERREICHBAR = 3;         // ber eine Verkettung von Listenobjekten erreichbar. 
  public static final int NICHTERREICHBAR = 4;    // gar nicht mehr erreichbar, Datenmll

  public static final int BUTTON_NOBUTTON = -1;   // kein Knopf auf Listenobjekt gedrckt 
  public static final int BUTTON_NULL = 1;        // Null-Button gedrckt
  public static final int BUTTON_CONTENT = 2;     // Inhalt-Button gedrckt
  
  private String inhalt;                          // Inhalt des Listenobjekts
  private int status = DIREKTERREICHBAR;          // Status der Erreichbarkeit (s. o.)
  
  // Ende Attribute

  // Anfang Methoden
  
  
  /**
   * Konstruktor fr ein neues Listenobjekt
   * @param zeiger der Zeiger, an dem das Objekt erstellt wird.
   * @param inhalt der Inhalt des neuen Listenobjekts
   * @param nachfolgerObjekt das referenzierte nachfolgeobjekt
   */
  public Listenobjekt(Zeiger zeiger, String inhalt, Listenobjekt nachfolgerObjekt, Optionen optionen) {
    super(zeiger.getX()+80,zeiger.getY()+50, optionen, nachfolgerObjekt);
    this.inhalt = inhalt;
    zeiger.setReferenz(this);
  }
  
  
  /**
   * Konstruktor fr ein neues Listenobjekt
   * @param x x-Position, an der das Objekt erstellt wird.
   * @param y y-Position, an der das Objekt erstellt wird.
   * @param inhalt der Inhalt des neuen Listenobjekts
   * @param nachfolgerObjekt das referenzierte nachfolgeobjekt
   */
  public Listenobjekt(int x, int y, String inhalt, Listenobjekt nachfolgerObjekt, Optionen optionen) {
    super(x, y, optionen, nachfolgerObjekt);
    this.inhalt = inhalt;
  }


  public String getInhalt() {
    return inhalt;
  }

  public void setInhalt(String inhalt) {
    this.inhalt = inhalt;
  }

  public void setX(int x) {    
    super.setX(x);
    if (getReferenz() == null) {
      getZeigerEnde().setX(x+130);
    }
  }

  public void setY(int y) {
    super.setY(y);
    if (getReferenz() == null) {
      getZeigerEnde().setY(y+25);
    }
  }
  
  public int getStatus() {
    return status;
  }
  
  public void setStatus(int status) {
    this.status = status;
  }
  
  
  /**
   * Methode zur Ermittlung, welcher Button innerhalb des Listenobjekts gedrckt wurde
   * @param absolutX die x-Koordinate auf dem Listenobjekt
   * @param absolutY die y-Koordinate auf dem Listenobjekt
   * @return die Knopf-Konstante (s. o.)
   */
  public int gibButton(int absolutX, int absolutY) {
    int x = absolutX - getX();
    int y = absolutY - getY();
    
    if (x>=5 && x<=15 && y>=5 && y<=15) {
      return BUTTON_CONTENT;      
    } else if (x>=85 && x<=95 && y>=20 && y<=30) {
      return BUTTON_NULL;      
    } 
    return BUTTON_NOBUTTON;
  }  
  
  
  /**
   * Methode zur Darstellung des Listenobjekts
   */
  public void darstellen(Graphics2D g) {
    g.setColor(Color.WHITE);
    g.fillRect(getX(),getY(),100,50);

    //Farbe und Linienstrke entsprechend der Erreichbarkeit des Listenobjekts
    switch (status) {
      case DIREKTERREICHBAR: g.setColor(Color.BLACK); break;
      case INDIREKTERREICHBAR: g.setColor(Color.DARK_GRAY); break;
      case ERREICHBAR: g.setColor(Color.GRAY); break;
      case NICHTERREICHBAR: g.setColor(Color.RED); break;
    }
    g.setStroke(new BasicStroke(4-status));
    // Kasten um das Objekt
    g.drawRect(getX(),getY(),100,50);

    // nun Innenleben zeichnen
    g.setStroke(new BasicStroke(1));
    // Trennlinie zwischen Inhalt und Zeiger
    g.drawLine(getX()+80,getY(),getX()+80,getY()+50);

    // Zeiger-Anfangs-Punkt
    g.fillOval(getX()+85,getY()+20,11,11);
    
    // Inhalt des Listenobjekts
    if (inhalt.equals("")) {
      g.setFont(new Font("Arial", Font.ITALIC, 12));
      g.drawString(optionen.labelEmpty, getX()+20, getY()+15);
    } else {
      g.setFont(new Font("Arial", Font.PLAIN, 12));
      g.drawString(inhalt, getX()+20, getY()+15);
    } 

    // Knopf fr Inhalt belegen zeichnen
    if (status == DIREKTERREICHBAR) {
      g.setColor(Color.GREEN);
      g.fillRect(getX()+5,getY()+5,11,11);
      g.setColor(Color.BLACK);
    } else {
      g.drawRect(getX()+5,getY()+5,10,10);
    }    
    g.drawLine(getX()+7,getY()+10,getX()+13,getY()+10);
    g.drawLine(getX()+10,getY()+7,getX()+13,getY()+10);
    g.drawLine(getX()+10,getY()+13,getX()+13,getY()+10);

  }
  

  // Ende Methoden
}
