import java.awt.*;
/**
  *
  * Objekte der Klasse stellen das Ende eines Zeigers dar. Er dient lediglich der
  * grafischen Veranschaulichung, wo die Referenz eines Zeigers oder eines
  * Listenenelements endet.
  *
  * @version 1.0 vom 23.11.2013
  * @author Daniel Garmann
  */

public class Punkt extends Darstellungsobjekt {
  
  // Anfang Attribute
  private Hauptspeicherobjekt besitzer;   // Der Besitzer des Zeiger-Ende-Punktes
  // Ende Attribute

  // Anfang Methoden
    
  /**
   * Konstruktor
   */
  public Punkt(int x, int y, Hauptspeicherobjekt besitzer, Optionen optionen) {
    super(x,y, optionen);
    this.besitzer = besitzer;
  }
  
  public Hauptspeicherobjekt getBesitzer() {
    return besitzer;
  }
  
  
  /**
   * Methode zur Darstellung des Endpunktes eines Pfeils
   */
  public void darstellen(Graphics2D g) {
    if (besitzer instanceof Zeiger) {
      g.setColor(Color.black);
    } else {
      // Je nach Erreichbarkeit eines Listenobjekts wird die Farbe eingestellt
      switch (((Listenobjekt)besitzer).getStatus()) {
        case Listenobjekt.DIREKTERREICHBAR: g.setColor(Color.BLACK); break;
        case Listenobjekt.INDIREKTERREICHBAR: g.setColor(Color.DARK_GRAY); break;
        case Listenobjekt.ERREICHBAR: g.setColor(Color.GRAY); break;
        case Listenobjekt.NICHTERREICHBAR: g.setColor(Color.RED); break;
      }
    }

    // Pfeilspitze zeichnen
    int vonX;
    int bisX = getX();
    int vonY = getBesitzer().getY() + 25;
    int bisY = getY();

    // Je nach Besitzer startet der Pfeil anders
    if (getBesitzer() instanceof Listenobjekt) {
      vonX = getBesitzer().getX() + 90;
    } else {
      vonX = getBesitzer().getX() + 25;
    }
    
    double l = 13.0; // Pfeilspitzenlnge
    double a = Math.PI/4 - Math.atan2((bisY - vonY),(bisX - vonX)); // Winkel des Pfeils
    double c = Math.cos(a)*l;
    double s = Math.sin(a)*l;
    
    // Polygon-Eckpunkte des Pfeils
    int[] xwerte = {bisX, (int)(bisX-s), (int)(bisX-c), bisX};
    int[] ywerte = {bisY, (int)(bisY-c), (int)(bisY+s), bisY};
    
    // Pfeilspitze zeichnen
    g.fillPolygon(xwerte,ywerte,xwerte.length);
    
    // Pfeillinie zeichnen
    g.drawLine(vonX, vonY, bisX, bisY);
    
    // Endpunkt des Pfeils hervorheben (daran zieht der Benutzer)
    g.fillOval(bisX-5,bisY-5,10,10);
    
    // Evtl. noch Referenzziel "null" ausgeben
    if (besitzer.getReferenz() == null) {
      g.setFont(new Font("Courier New", Font.BOLD, 12));
      g.drawString(optionen.labelNull,getX()+10,getY()+5);
    }
            
  }
  
  // Ende Methoden
}
