import java.awt.*;
/**
  *
  * Objekte der Klasse stellen einen Zeiger des Hauptspeichers dar. Ein Zeiger
  * referenziert immer Listenobjekte oder zeigt auf null. Mithilfe des Zeigers
  * knnen neue Listenelemente erstellt werden.
  *
  * @version 1.0 vom 04.01.2014
  * @author Daniel Garmann
  */

public class Zeiger extends Hauptspeicherobjekt{
  
  // Anfang Attribute
  public static final int BUTTON_NOBUTTON = -1;      // kein Knopf auf Listenobjekt gedrckt 
  public static final int BUTTON_PLUS = 1;           // Knopf zur Erstellung eines Listenobjekts gedrckt 
  public static final int BUTTON_NULL = 2;           // Null-Knopf gedrckt
  public static final int BUTTON_DISPOSE = 3;        // Lschen-Knopf gedrckt (Zeiger entfernen)
  
  private String name;                               // Zeigername
  private boolean statisch = false;                  // Flag, ob der Zeiger gelscht werden kann.
  // Ende Attribute

  // Anfang Methoden
  
  /**
   * Konstruktor fr einen neuen Zeiger
   * @param x x-Position, an der der Zeiger erstellt wird.
   * @param y y-Position, an der der Zeiger erstellt wird.
   * @param name der Name des neuen Zeigers
   * @param neueReferenz das referenzierte Listenobjekt
   */
  public Zeiger(int x, int y, String name, Listenobjekt neueReferenz, Optionen optionen) {
    super(x,y, optionen, neueReferenz);
    this.name = name;
    statisch = false;
  }

  public void setStatisch(boolean wert) {
    statisch = wert;
  }
  
  public boolean istStatisch() {
    return statisch;
  }
  
  public String getName() {
    return name;
  }

  public void setName(String name) {
    this.name = name;
  }

  public void setX(int x) {
    super.setX(x);
    if (getReferenz() == null) {
      getZeigerEnde().setX(x+80);
    }
  }

  public void setY(int y) {
    super.setY(y);
    if (getReferenz() == null) {
      getZeigerEnde().setY(y+25);
    }
  }
  
  /**
   * Methode zur Ermittlung, welcher Button innerhalb des Zeigers gedrckt wurde
   * @param absolutX die x-Koordinate auf dem Zeiger
   * @param absolutY die y-Koordinate auf dem Zeiger
   * @return die Knopf-Konstante (s. o.)
   */
  public int gibButton(int absolutX, int absolutY) {
    int x = absolutX - getX();
    int y = absolutY - getY();
    
    if (x>=20 && x<=45 && y>=35 && y<=45) {
      return BUTTON_PLUS;
    } else if (x>=20 && x<=30 && y>=20 && y<=30) {
      return BUTTON_NULL;
    } else if (x>=35 && x<=45 && y>=5 && y<=15 && statisch == false) {
      return BUTTON_DISPOSE;
    }    
    return BUTTON_NOBUTTON;
  }  
  
  /**
   * Methode zur Darstellung des Listenobjekts
   */
  public void darstellen(Graphics2D g) {    
    // erste einen Kasten um den Zeiger
    g.setColor(Color.WHITE);
    g.fillRect(getX(),getY(),50,50);
    g.setColor(Color.BLACK);
    g.drawRect(getX(),getY(),50,50);
    
    // dann den Zeigernamen oberhalb des Zeigers
    g.setFont(new Font("Arial", Font.PLAIN, 12));
    g.drawString(name,getX(),getY()-5);
    
    // und den Startpunkt des Zeigers
    g.fillOval(getX()+20,getY()+20,11,11);

    // den Plus-Button zur Erstellung eines Listenobjekts
    g.setColor(Color.GREEN);
    g.fillRect(getX()+20,getY()+35,26,11);
    g.setColor(Color.BLACK);
//    g.drawLine(getX()+37,getY()+40,getX()+43,getY()+40);
//    g.drawLine(getX()+40,getY()+37,getX()+40,getY()+43);
    g.setFont(new Font("Courier New", Font.BOLD, 10));
    g.drawString(optionen.labelNew,getX()+23,getY()+43);

    // und den Lschen-Button zur Lschung des Zeigers
    if (statisch == false) {
      g.setColor(Color.RED);
      g.fillRect(getX()+35,getY()+5,11,11);
      g.setColor(Color.WHITE);
      g.drawLine(getX()+37,getY()+7,getX()+43,getY()+13);
      g.drawLine(getX()+37,getY()+13,getX()+43,getY()+7);
    }
  }


  // Ende Methoden
}
